/**
 * ECS root file
 * 
 * Contributors:
 * Brychan Dempsey - brychand@hotmail.com
 * 
 * References:
 * https://code.austinmorlan.com/austin/ecs - 'A simple C++ Entity Component System' - released under MIT licence
 *  
 */

import java.util.*;

public class ECS {
    EntityManager entityManager;
    ComponentManager componentManager;
    SystemManager systemManager;

    public ECS(){
        entityManager = new EntityManager();
        componentManager = new ComponentManager();
        systemManager = new SystemManager();
    }


    Integer createEntity(){
        return entityManager.addEntity();
    }

    void destroyEntity(int entity){
        entityManager.removeEntity(entity);
        componentManager.entityDestroyed(entity);
        systemManager.entityDestroyed(entity);
    }

    void registerComponent(String name){
        componentManager.registerComponent(name);
    }

    void addComponent(int entity, String componentName, Object component){
        componentManager.addComponentToEntity(componentName, component, entity);
        entityManager.registerComponent(componentManager.getComponentIndex(componentName), entity);
        systemManager.entitySignatureChanged(entity, entityManager.getRegistrations(entity));
    }

    void removeComponent(int entity, String componentName){
        componentManager.removeComponentFromEntity(componentName, entity);
        entityManager.unregisterComponent(componentManager.getComponentIndex(componentName), entity);
        systemManager.entitySignatureChanged(entity, entityManager.getRegistrations(entity));
    }

    Object getComponentData(int entity, String componentName){
        return componentManager.getComponentData(entity, componentName);
    }

    void setSystemSignature(String system, BitSet signature){
        systemManager.setSignature(system, signature);
    }
}