import java.util.BitSet;
import java.util.Map;

class SystemManager{
    Map<String, BitSet> signatures = new HashMap<>();
    Map<String, ECSSystem> systems = new HashMap<>();


    public SystemManager(ECS baseECS){
        this.baseECS = baseECS;
        systemIndex = 0;
    }
    // Registering the system adds it to the array of systems.
    // In Austin Morlan's implementation, this also creates an instance of the
    // system that can be called from the main thread.
    // It returns this object.
    // In Java, we need to initialise the object first (before this is called),
    // then register that object. Enactment of the system is performed on that 
    // instance.
    // I.e., create an object that represents the system class; then store that class in the system
    // table.
    public boolean registerSystem(String system, ECSSystem action){
        if (systems.containsKey(system)){
            System.err.println("System already registered");
            return false;
        }
        systems.put(system, action);
        return true;
    }

    public void setSignature(String system, BitSet registrations){
        signatures.put(system, registrations);
    }

    public void entityDestroyed(int entity){
        for (String key : systems.keySet()) {
            systems.get(key).entities.remove(entity);
        }
    }

    public void entitySignatureChanged(int entity, BitSet entitySignature){
        for (String key : systems.keySet()) {
            if (entitySignature.and(signatures.get(key)) == signatures.get(key)){ // Bitwise check if the entity is subscribed to this system
                systems.get(key).entities.add(entity);
            }
            else{
                systems.get(key).entities.remove(entity);
            }
        }
    }
}